function val_crit = eval_crit(crit, data, n, phat, Yout, noz)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
%crit   = (1) SBC; (2) FPE; (3) RNML; (4) AICC; (5) EBIC
%       = (6) EBICFD; (7) RNMLFD
%data   = measurements
%n      = no. manifest var. plus latent var.
%phat   = estimated VAR order
%Yout   = causal part of estimated matrix polynomial $S^{-1}$ (IShat)
%noz    = number of zeros in the mask (below the main diagonal)
%Output:
%val_crit   = criterion value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%N = sample size;
%K = no. of manifest variables
[N, K] = size(data);

%Perform spectral factorization
[A,~] = spectral_fact(n, phat, Yout);

%Pre-multiply the coefficients of the model
%Keep only the part corresponding to manifest var.
B = zeros(K, (phat+1)*K);
for ind = 1:phat+1,
    temp = A{1}\A{ind};
    B(:,(ind-1)*K+1:ind*K) = temp(1:K,1:K);
end

%Compute log(det(Sigma))
T = N - phat;
Ut= zeros(T, K*(phat+1));
for j = 1:phat+1
    Ut(:,K*(j-1)+1:j*K) = data(phat+2-j:N-j+1,:);
end
temp  = Ut*B';
Sigma = temp'*temp/T;
logdetSigma = log(det(Sigma));

%Compute R2 (modified R)
Y = data(phat+1:end,:);
R2= trace(Y'*Y - T*Sigma)/T;

%No. of param. [Songsiri et al., book chapter, p. 106]
nop = K*(K+1)/2 - noz + phat*(K^2-2*noz);
nop1= nop/K;

%Here we calculate the values of the criteria
%SBC
if crit == 1,
    val_crit = T*logdetSigma + nop*log(T);
    %FPE
elseif crit == 2,
    val_crit = K*log((T+nop1)/(T-nop1))+logdetSigma;
    %RNML
elseif crit == 3,
    if R2 <= 0
        val_crit = Inf;
    else
        ell     = nop1;
        term    = zeros(4,1);
        term(1) = ((T-ell-K+1)/2)*logdetSigma;
        term(2) = -sum(gammaln((T-ell:-1:T-ell+1-K)/2));
        term(3) = -gammaln(ell*K/2);
        term(4) = ((ell*K)/2)*log(R2);
        val_crit = sum(term);
    end
    %AICC
elseif crit == 4,
    val_crit = T*logdetSigma + (2*nop*T)/(T-nop-1);
    %EBIC
elseif crit == 5,
    kbar     = K*(K-1)/2;
    supterm  = gammaln(kbar+1) - gammaln(noz+1) - gammaln(kbar-noz+1);
    coeff    = 1;
    val_crit = T*logdetSigma + nop*log(T) + 2*coeff*supterm;
    %EBICFD
elseif crit == 6,
    gam      = 1;
    val_crit = T*logdetSigma + nop*log(T) + 4*nop*gam*log(K);
    %RNMLFD
elseif crit == 7,
    if R2 <= 0
        val_crit = Inf;
    else
        ell     = nop1;
        term    = zeros(4,1);
        term(1) = ((T-ell-K+1)/2)*logdetSigma;
        term(2) = -sum(gammaln((T-ell:-1:T-ell+1-K)/2));
        term(3) = -gammaln(ell*K/2);
        term(4) = ((ell*K)/2)*log(R2);
        val_crit = sum(term);
        gam      = 1;
        val_crit = val_crit + 2*nop*gam*log(K);
    end
else
    fprintf('Error: Incorrect ITC!\n');
    val_crit = Inf;
end

end %function


